class EWBridge { 
    constructor () {
        this.uniqueId = 1
        this.callbacks = {}
        this.instance = null;
        window.nativeCallback = EWBridge.nativeCallback
        console.log('实例化 EWBridge',this.instance)
    }


    static getInstance() {
        if (!this.instance) {
            this.instance = new EWBridge();
            return this.instance
        }
        return this.instance;
    }

    // native回调的地方
    static nativeCallback (response) {
        var obj = JSON.parse(response)
        var key = obj['uniqueId']
        var callback = EWBridge.getInstance().loadCallback(key)
        if (callback) {
            callback(obj['parameters'])
        }
    }
    
    // 保存回调
    saveCallback (callback) {

        if (callback) {
            this.callbacks[this.uniqueId] = callback;
            this.uniqueId++
        }
    }
    // 获取回调
    loadCallback (key) {
        
        if (!this.callbacks.hasOwnProperty(key)) {
            return null
        }
        var callback = this.callbacks[key]
        delete this.callbacks[key]
        return callback
    }
    
    objToString (obj) {
        var data = {
            'uniqueId': this.uniqueId,
            'parameters': obj,
            'callback':'nativeCallback'
        }
        return JSON.stringify(data)
    }

    detect() {
        var equipmentType = "";
        var agent = navigator.userAgent.toLowerCase();
        var android = agent.indexOf("android");
        var iphone = agent.indexOf("iphone");
        var ipad = agent.indexOf("ipad");
        // let isUseBridge = getCookie('isUseBridge');
        if(android != -1){
            equipmentType = "android";
        }
        if(iphone != -1 || ipad != -1){
            equipmentType = "ios";
        }
        // if(!isUseBridge) equipmentType = "pc";
        return equipmentType;
    }

    getInfo () {
        return new Promise((resolve, reject) => {
            var str = this.objToString({})
            this.saveCallback(data => {
                if (data['code'] == '1') {
                    resolve(data['data'])
                } else {
                    reject(data['message'])
                }
            })
            if (this.detect() == 'ios') {              
                window.webkit.messageHandlers.getInfo.postMessage(str)
            } else if(this.detect() == 'android') {
                window.bridge.getInfo(str)
            }
        })


        
    }

    signTransact (chainId, endpoint, tx) {
        return new Promise((resolve, reject) => {
            var str = this.objToString({'chainId': chainId, 'endpoint': endpoint, 'tx': tx})
            this.saveCallback(data => {
                if (data['code'] == '1') {
                    resolve(data['data'])
                } else {
                    reject(data['message'])
                }
            })
            if (this.detect() == 'ios') {
                window.webkit.messageHandlers.signTransact.postMessage(str)
            } else {
                window.bridge.signTransact(str)
            }
        })
    }
    
}
module.exports = EWBridge;