const {
    Api,
    JsonRpc,
    ecc
} = require('lcnjs')
const Bridge = require('./EWBridge').getInstance();
function arrayToHex(data) {
    let result = '';
    for (const x of data) {
        result += ('00' + x.toString(16)).slice(-2);
    }
    return result;
}
const cfg = {
    expireSeconds: 60,
    blocksBehind: 3,
    broadcast: false,
    sign: false
}

class ScatterDapp {

    static async getInfo() {
        return Bridge.getInfo()
    }

    static async loadScatter(callback) {
        let lcn, account
        return new Promise(async (resolve, reject) => {
            try {
                account = await Bridge.getInfo()
                if (account) {
                    lcn = this.newApi(account)
                    return resolve(lcn)
                } else {
                    return reject()
                }
            } catch (error) {
                return reject(error)
            }
        })

    }

    static newApi(account) {
        let lcn
        const { chainId, node_path } = account
        const transact = async (transaction) => {
            const data = await lcn.transact(transaction, cfg)
            const serializedTransaction = arrayToHex(data.serializedTransaction)
            const signatures = await Bridge.signTransact(chainId, node_path, serializedTransaction)
            const pk_tx = {
                signatures, // 签名列表
                serializedTransaction: data.serializedTransaction
            }
            return lcn.push_transaction(pk_tx)
        }
        const endpoint = account && account.endpointsrvlist ? account.endpointsrvlist[0] : 'http://39.106.5.40/chain'
        const rpc = new JsonRpc(endpoint, {
            fetch
        })
        lcn = new Api({
            rpc,
            textDecoder: new TextDecoder(),
            textEncoder: new TextEncoder()
        })
        lcn.transact = (arg) => transact(arg)
        return lcn
    }
}

module.exports = ScatterDapp